@@ -15,9 +15,9 @@ possible by providing a class responsible to get astng representation
 from various source and using a cache of built modules)
 
 :author:    Sylvain Thenault
-:copyright: 2003-2007 LOGILAB S.A. (Paris, FRANCE)
+:copyright: 2003-2009 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-:copyright: 2003-2007 Sylvain Thenault
+:copyright: 2003-2009 Sylvain Thenault
 :contact:   mailto:thenault@gmail.com
 """
 
@@ -27,13 +27,12 @@ import sys
 import os
 from os.path import dirname, basename, abspath, join, isdir, exists
 
-from logilab.common.cache import Cache
 from logilab.common.modutils import NoSourceFile, is_python_source, \
      file_from_modpath, load_module_from_name, \
-     get_module_files, get_source_file
+     get_module_files, get_source_file, zipimport
 from logilab.common.configuration import OptionsProviderMixIn
 
-from logilab.astng import ASTNGBuildingException, Instance, nodes
+from logilab.astng import ASTNGBuildingException, nodes, infutils
 
 def astng_wrapper(func, modname):
     """wrapper to give to ASTNGManager.project_from_files"""
@@ -54,6 +53,22 @@ def safe_repr(obj):
     except:
         return '???'
 
+def zip_import_data(filepath):
+    if zipimport is None:
+        return None, None
+    for ext in ('.zip', '.egg'):
+        try:
+            eggpath, resource = filepath.split(ext + '/', 1)
+        except ValueError:
+            continue
+        try:
+            importer = zipimport.zipimporter(eggpath + ext)
+            return importer.get_source(resource), resource.replace('/', '.')
+        except:
+            continue
+    return None, None
+
+        
 class ASTNGManager(OptionsProviderMixIn):
     """the astng manager, responsible to build astng from files
      or modules.
@@ -74,8 +89,9 @@ class ASTNGManager(OptionsProviderMixIn):
                  'help' : 'set the project name.'}),
                )
     brain = {}
-    def __init__(self):
-        self.__dict__ = ASTNGManager.brain
+    def __init__(self, borg=True):
+        if borg:
+            self.__dict__ = ASTNGManager.brain
         if not self.__dict__:
             OptionsProviderMixIn.__init__(self)
             self._cache = None
@@ -108,10 +124,11 @@ class ASTNGManager(OptionsProviderMixIn):
                 try:
                     from logilab.astng.builder import ASTNGBuilder
                     astng = ASTNGBuilder(self).file_build(filepath, modname)
-                except SyntaxError:
+                except (SyntaxError, KeyboardInterrupt, SystemExit):
                     raise
                 except Exception, ex:
                     if __debug__:
+                        print 'error while building astng for', filepath
                         import traceback
                         traceback.print_exc()
                     msg = 'Unable to load module %s (%s)' % (modname, ex)
@@ -133,6 +150,19 @@ class ASTNGManager(OptionsProviderMixIn):
             os.chdir(dirname(context_file))
         try:
             filepath = self.file_from_module_name(modname, context_file)
+            if filepath is not None and not is_python_source(filepath):
+                try:
+                    return self._cache[filepath]
+                except KeyError:
+                    data, zmodname = zip_import_data(filepath)
+                    if data is not None:
+                        from logilab.astng.builder import ASTNGBuilder
+                        try:
+                            astng = ASTNGBuilder(self).string_build(data, zmodname, filepath)
+                        except (SyntaxError, KeyboardInterrupt, SystemExit):
+                            raise
+                        self._cache[filepath] = astng
+                        return astng
             if filepath is None or not is_python_source(filepath):
                 try:
                     module = load_module_from_name(modname)
@@ -221,7 +251,7 @@ class ASTNGManager(OptionsProviderMixIn):
         modastng = self.astng_from_module_name(modname)
         for infered in modastng.igetattr(name, context):
             if klass is not obj and isinstance(infered, nodes.Class):
-                infered = Instance(infered)
+                infered = infutils.Instance(infered)
             yield infered
 
     def project_from_files(self, files, func_wrapper=astng_wrapper,
@@ -376,7 +406,7 @@ class Project:
     def get_module(self, name):
         return self.locals[name]
 
-    def getChildNodes(self):
+    def get_children(self):
         return self.modules
 
     def __repr__(self):
