@@ -14,9 +14,9 @@
 (build_* functions) or from living object (object_build_* functions)
 
 :author:    Sylvain Thenault
-:copyright: 2003-2007 LOGILAB S.A. (Paris, FRANCE)
+:copyright: 2003-2009 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-:copyright: 2003-2007 Sylvain Thenault
+:copyright: 2003-2009 Sylvain Thenault
 :contact:   mailto:thenault@gmail.com
 """
 
@@ -27,11 +27,10 @@ from inspect import getargspec
 
 from logilab.astng import nodes
 
-def attach___dict__(node):
-    """attach the __dict__ attribute to Class and Module objects"""
-    dictn = nodes.Dict([])
-    dictn.parent = node
-    node.locals['__dict__'] = [dictn]
+    
+def _attach_local_node(parent, node, name):
+    node.name = name # needed by add_local_node
+    parent.add_local_node(node)
 
 _marker = object()
 
@@ -44,41 +43,25 @@ def attach_dummy_node(node, name, object=_marker):
     _attach_local_node(node, enode, name)
 
 nodes.EmptyNode.has_underlying_object = lambda self: self.object is not _marker
-
+    
 def attach_const_node(node, name, value):
     """create a Const node and register it in the locals of the given
     node with the specified name
     """
-    _attach_local_node(node, nodes.Const(value), name)
+    if not name in node.special_attributes:
+        _attach_local_node(node, nodes.const_factory(value), name)
 
-if sys.version_info < (2, 5):
-    def attach_import_node(node, modname, membername):
-        """create a From node and register it in the locals of the given
-        node with the specified name
-        """
-        _attach_local_node(node,
-                           nodes.From(modname, ( (membername, None), ) ),
-                           membername)
-else:
-    def attach_import_node(node, modname, membername):
-        """create a From node and register it in the locals of the given
-        node with the specified name
-        """
-        _attach_local_node(node,
-                           nodes.From(modname, ( (membername, None), ), 0),
-                           membername)
-    
-def _attach_local_node(parent, node, name):
-    node.name = name # needed by add_local_node
-    node.parent = parent
-    node.lineno = 1
-    parent.add_local_node(node)
+def attach_import_node(node, modname, membername):
+    """create a From node and register it in the locals of the given
+    node with the specified name
+    """
+    _attach_local_node(node, nodes.import_from_factory(modname, membername),
+                       membername)
 
 
 def build_module(name, doc=None):
     """create and initialize a astng Module node"""
-    node = nodes.Module(doc, nodes.Stmt([]))
-    node.node.parent = node
+    node = nodes.module_factory(doc)
     node.name = name
     node.pure_python = False
     node.package = False
@@ -86,67 +69,32 @@ def build_module(name, doc=None):
     node.globals = node.locals = {}
     return node
 
-def build_class(name, basenames=None, doc=None):
+def build_class(name, basenames=(), doc=None):
     """create and initialize a astng Class node"""
-    klass = nodes.Class(name, [], doc, nodes.Stmt([]))
-    bases = [nodes.Name(base) for base in basenames]
-    for base in bases:
-        base.parent = klass
-    klass.basenames = basenames
-    klass.bases = bases
-    klass.code.parent = klass
-    klass.locals = {}
-    klass.instance_attrs = {}
-    for name, value in ( ('__name__', name),
-                         #('__module__', node.root().name),
-                         ):
-        const = nodes.Const(value)
-        const.parent = klass
-        klass.locals[name] = [const]
-    return klass
+    node = nodes.class_factory(name, basenames, doc)
+    node.locals = {}
+    node.instance_attrs = {}
+    return node
 
-# introduction of decorators has changed the Function initializer arguments
-if sys.version_info >= (2, 4):
-    try:
-        from compiler.ast import Decorators as BaseDecorators
-        class Decorators(BaseDecorators):
-            def __init__(self):
-                BaseDecorators.__init__(self, [], 0)
-    except ImportError:
-        Decorators = list
-        
-    def build_function(name, args=None, defaults=None, flag=0, doc=None):
-        """create and initialize a astng Function node"""
-        args, defaults = args or [], defaults or []
-        # first argument is now a list of decorators
-        func = nodes.Function(Decorators(), name, args, defaults, flag, doc,
-                              nodes.Stmt([]))
-        func.code.parent = func
-        func.locals = {}
-        if args:
-            register_arguments(func, args)
-        return func
-    
-else:    
-    def build_function(name, args=None, defaults=None, flag=0, doc=None):
-        """create and initialize a astng Function node"""
-        args, defaults = args or [], defaults or []
-        func = nodes.Function(name, args, defaults, flag, doc, nodes.Stmt([]))
-        func.code.parent = func
-        func.locals = {}
-        if args:
-            register_arguments(func, args)
-        return func
+def build_function(name, args=None, defaults=None, flag=0, doc=None):
+    """create and initialize a astng Function node"""
+    args, defaults = args or [], defaults or []
+    # first argument is now a list of decorators
+    func = nodes.function_factory(name, args, defaults, flag, doc)
+    func.locals = {}
+    if args:
+        register_arguments(func)
+    return func
 
 
-def build_name_assign(name, value):
-    """create and initialize an astng Assign for a name assignment"""
-    return nodes.Assign([nodes.AssName(name, 'OP_ASSIGN')], nodes.Const(value))
+# def build_name_assign(name, value):
+#     """create and initialize an astng Assign for a name assignment"""
+#     return nodes.Assign([nodes.AssName(name, 'OP_ASSIGN')], nodes.Const(value))
 
-def build_attr_assign(name, value, attr='self'):
-    """create and initialize an astng Assign for an attribute assignment"""
-    return nodes.Assign([nodes.AssAttr(nodes.Name(attr), name, 'OP_ASSIGN')],
-                        nodes.Const(value))
+# def build_attr_assign(name, value, attr='self'):
+#     """create and initialize an astng Assign for an attribute assignment"""
+#     return nodes.Assign([nodes.AssAttr(nodes.Name(attr), name, 'OP_ASSIGN')],
+#                         nodes.Const(value))
 
 if sys.version_info < (2, 5):
     def build_from_import(fromname, names):
@@ -157,17 +105,23 @@ else:
         """create and intialize an astng From import statement"""
         return nodes.From(fromname, [(name, None) for name in names], 0)
 
-def register_arguments(node, args):
+def register_arguments(func, args=None):
     """add given arguments to local
     
     args is a list that may contains nested lists
     (i.e. def func(a, (b, c, d)): ...)
     """
+    if args is None:
+        args = func.args.args
+        if func.args.vararg:
+            func.set_local(func.args.vararg, func.args)
+        if func.args.kwarg:
+            func.set_local(func.args.kwarg, func.args)
     for arg in args:
-        if type(arg) is type(''):
-            node.set_local(arg, node)
+        if isinstance(arg, nodes.Name):
+            func.set_local(arg.id, arg)
         else:
-            register_arguments(node, arg)
+            register_arguments(func, arg.elts)
 
 
 def object_build_class(node, member):
@@ -194,9 +148,14 @@ def object_build_methoddescriptor(node, member):
     """create astng for a living method descriptor object"""
     # FIXME get arguments ?
     func = build_function(member.__name__, doc=member.__doc__)
-    # set argnames to None to notice that we have no information, not
+    # set node's arguments to None to notice that we have no information, not
     # and empty argument list
-    func.argnames = None 
+    func.args = nodes.Arguments()
+    # arg values are not defined on an _ast node
+    func.args.args = None
+    func.args.defaults = None
+    func.args.vararg = None
+    func.args.kwarg = None
     node.add_local_node(func)
 
 def _base_class_object_build(node, member, basenames, name=None):
@@ -230,5 +189,5 @@ def _base_class_object_build(node, member, basenames, name=None):
 __all__ = ('register_arguments',  'build_module', 
            'object_build_class', 'object_build_function', 
            'object_build_datadescriptor', 'object_build_methoddescriptor',
-           'attach___dict__', 'attach_dummy_node',
+           'attach_dummy_node',
            'attach_const_node', 'attach_import_node')
